@@ -1,7 +1,7 @@
 package CPAN::SQLite::CPANMeta;
 
 our $DATE = '2015-01-09'; # DATE
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 use 5.010001;
 use strict;
@@ -30,6 +30,21 @@ $SPEC{':package'} = {
     summary => 'Index and query CPAN Meta information in CPAN::SQLite database',
 };
 
+sub _connect_db {
+    require DBI;
+
+    my %args = @_;
+
+    my $cpan    = $args{cpan};
+    my $db_dir  = $args{db_dir} // $cpan;
+    my $db_name = $args{db_name} // 'cpandb.sql';
+
+    my $db_path = "$db_dir/$db_name";
+    $log->tracef("Connecting to SQLite database at %s ...", $db_path);
+    DBI->connect("dbi:SQLite:dbname=$db_path", undef, undef,
+                 {RaiseError=>1});
+}
+
 sub _parse_json {
     my $content = shift;
 
@@ -107,14 +122,10 @@ sub index_cpan_meta {
 
     my %args = @_;
 
-    my $cpan    = $args{cpan} or return [412, "Please specify 'cpan'"];
-    my $db_dir  = $args{db_dir} // $cpan;
-    my $db_name = $args{db_name} // 'cpandb.sql';
+    my $cpan = $args{cpan} or return [400, "Please specify 'cpan'"];
+
+    my $dbh  = _connect_db(%args);
 
-    my $db_path = "$db_dir/$db_name";
-    $log->tracef("Connecting to SQLite database at %s ...", $db_path);
-    my $dbh = DBI->connect("dbi:SQLite:dbname=$db_path", undef, undef,
-                           {RaiseError=>1});
     $dbh->do("CREATE INDEX IF NOT EXISTS ix_dists_dist_file ON dists(dist_file)");
     $dbh->do("CREATE TABLE IF NOT EXISTS files (
   file_id INTEGER NOT NULL PRIMARY KEY,
@@ -340,8 +351,10 @@ ORDER BY dist_file
     } # for file
 
     $dbh->commit if $after_begin;
-    undef $sth_insdist;
     undef $sth_insfile;
+    undef $sth_seldist;
+    undef $sth_insdist;
+    undef $sth_selmod;
     undef $sth_insdep;
     undef $sth;
 
@@ -407,63 +420,141 @@ ORDER BY module");
     [200, "OK", \@res];
 }
 
+sub _get_revdeps {
+    my ($mod, $dbh) = @_;
+
+    $log->tracef("Finding reverse dependencies for module %s ...", $mod);
+
+    # first, check that module is listed
+    my ($mod_id) = $dbh->selectrow_array("SELECT mod_id FROM mods WHERE mod_name=?", {}, $mod)
+        or return [404, "No such module: $mod"];
+
+    # get all dists that depend on that module
+    my $sth = $dbh->prepare("SELECT
+  (SELECT dist_name FROM dists WHERE dp.dist_id=dists.dist_id) AS dist,
+  (SELECT dist_vers FROM dists WHERE dp.dist_id=dists.dist_id) AS dist_version,
+  -- phase,
+  -- rel,
+  version req_version
+FROM deps dp
+WHERE mod_id=?
+ORDER BY dist");
+    $sth->execute($mod_id);
+    my @res;
+    while (my $row = $sth->fetchrow_hashref) {
+        #next unless $phase eq 'ALL' || $row->{phase} eq $phase;
+        #next unless $rel   eq 'ALL' || $row->{rel}   eq $rel;
+        #delete $row->{phase} unless $phase eq 'ALL';
+        #delete $row->{rel}   unless $rel   eq 'ALL';
+        push @res, $row;
+    }
+
+    [200, "OK", \@res];
+}
+
+my $_complete_mod = sub {
+    my %args = @_;
+
+    my $word = $args{word} // '';
+
+    my $cmdline = $args{cmdline} or return undef;
+    my $r = $args{r};
+
+    # force read config file, because by default it is turned off when in
+    # completion
+    $r->{read_config} = 1;
+    my $res = $cmdline->parse_argv($r);
+
+    my $dbh;
+    eval { $dbh = _connect_db(%{$res->[2]}) };
+
+    # if we can't connect (probably because --cpan is not set yet), bail
+    if ($@) {
+        $log->tracef("[comp] can't connect to db, bailing: %s", $@);
+        return undef;
+    }
+
+    my $sth = $dbh->prepare(
+        "SELECT mod_name FROM mods WHERE mod_name LIKE ? ORDER BY mod_name");
+    $sth->execute($word . '%');
+
+    # XXX follow Complete::OPT_CI
+
+    my @res;
+    while (my ($mod) = $sth->fetchrow_array) {
+        # only complete one level deeper at a time
+        if ($mod =~ /:\z/) {
+            next unless $mod =~ /\A\Q$word\E:*\w+\z/i;
+        } else {
+            next unless $mod =~ /\A\Q$word\E\w*(::\w+)?\z/i;
+        }
+        push @res, $mod;
+    }
+
+    \@res;
+};
+
+my %mod_args = (
+    module => {
+        schema => 'str*',
+        req => 1,
+        pos => 0,
+        completion => $_complete_mod,
+    },
+);
+
+my %deps_args = (
+    phase => {
+        schema => ['str*' => {
+            in => [qw/develop configure build runtime test ALL/],
+        }],
+        default => 'runtime',
+    },
+    rel => {
+        schema => ['str*' => {
+            in => [qw/requires recommends suggests conflicts ALL/],
+        }],
+        default => 'requires',
+    },
+    level => {
+        summary => 'Recurse for a number of levels (-1 means unlimited)',
+        schema  => 'int*',
+        default => 1,
+        cmdline_aliases => {
+            l => {},
+            R => {
+                summary => 'Recurse (alias for `--level -1`)',
+                is_flag => 1,
+                code => sub { $_[0]{level} = -1 },
+            },
+        },
+    },
+    include_core => {
+        summary => 'Include Perl core modules',
+        'summary.alt.bool.not' => 'Exclude Perl core modules',
+        schema  => 'bool',
+        default => 0,
+    },
+    perl_version => {
+        summary => 'Set base Perl version for determining core modules',
+        schema  => 'str*',
+        default => "$^V",
+        cmdline_aliases => {V=>{}},
+    },
+);
+
 $SPEC{'deps_cpan_meta'} = {
     v => 1.1,
     summary => 'Query dependency from CPAN::SQLite database',
     args => {
         %common_args,
-        module => {
-            schema => 'str*',
-            req => 1,
-            pos => 0,
-        },
-        phase => {
-            schema => ['str*' => {
-                in => [qw/develop configure build runtime test ALL/],
-            }],
-            default => 'runtime',
-        },
-        rel => {
-            schema => ['str*' => {
-                in => [qw/requires recommends suggests conflicts ALL/],
-            }],
-            default => 'requires',
-        },
-        level => {
-            summary => 'Recurse for a number of levels (-1 means unlimited)',
-            schema  => 'int*',
-            default => 1,
-            cmdline_aliases => {
-                l => {},
-                R => {
-                    summary => 'Recurse (alias for `--level -1`)',
-                    is_flag => 1,
-                    code => sub { $_[0]{level} = -1 },
-                },
-            },
-        },
-        include_core => {
-            summary => 'Include Perl core modules',
-            'summary.alt.bool.not' => 'Exclude Perl core modules',
-            schema  => 'bool',
-            default => 0,
-        },
-        perl_version => {
-            summary => 'Set base Perl version for determining core modules',
-            schema  => 'str*',
-            default => "$^V",
-            cmdline_aliases => {V=>{}},
-        },
+        %deps_args,
     },
 };
 sub deps_cpan_meta {
-    require DBI;
-
     my %args = @_;
 
-    my $cpan    = $args{cpan} or return [412, "Please specify 'cpan'"];
-    my $db_dir  = $args{db_dir} // $cpan;
-    my $db_name = $args{db_name} // 'cpandb.sql';
+    my $cpan    = $args{cpan} or return [400, "Please specify 'cpan'"];
     my $mod     = $args{module};
     my $phase   = $args{phase} // 'runtime';
     my $rel     = $args{rel} // 'requires';
@@ -471,10 +562,7 @@ sub deps_cpan_meta {
     my $level   = $args{level} // 1;
     my $include_core = $args{include_core} // 0;
 
-    my $db_path = "$db_dir/$db_name";
-    $log->tracef("Connecting to SQLite database at %s ...", $db_path);
-    my $dbh = DBI->connect("dbi:SQLite:dbname=$db_path", undef, undef,
-                           {RaiseError=>1});
+    my $dbh     = _connect_db(%args);
 
     my $res = _get_prereqs($mod, $dbh, {}, 1, $level, $phase, $rel, $include_core, $plver);
 
@@ -492,29 +580,18 @@ $SPEC{'revdeps_cpan_meta'} = {
     summary => 'Query reverse dependencies from CPAN::SQLite database',
     args => {
         %common_args,
-        module => {
-            schema => 'str*',
-            req => 1,
-            pos => 0,
-        },
+        %mod_args,
     },
 };
 sub revdeps_cpan_meta {
-    require DBI;
-
     my %args = @_;
 
-    my $cpan    = $args{cpan} or return [412, "Please specify 'cpan'"];
-    my $db_dir  = $args{db_dir} // $cpan;
-    my $db_name = $args{db_name} // 'cpandb.sql';
+    my $cpan    = $args{cpan} or return [400, "Please specify 'cpan'"];
     my $mod     = $args{module};
 
-    my $db_path = "$db_dir/$db_name";
-    $log->tracef("Connecting to SQLite database at %s ...", $db_path);
-    my $dbh = DBI->connect("dbi:SQLite:dbname=$db_path", undef, undef,
-                           {RaiseError=>1});
+    my $dbh     = _connect_db(%args);
 
-    [501, "Not yet implemented"];
+    _get_revdeps($mod, $dbh);
 }
 
 1;
@@ -532,11 +609,11 @@ CPAN::SQLite::CPANMeta - Index and query CPAN Meta information in CPAN::SQLite d
 
 =head1 VERSION
 
-This document describes version 0.03 of CPAN::SQLite::CPANMeta (from Perl distribution CPAN-SQLite-CPANMeta), released on 2015-01-09.
+This document describes version 0.04 of CPAN::SQLite::CPANMeta (from Perl distribution CPAN-SQLite-CPANMeta), released on 2015-01-09.
 
 =head1 SYNOPSIS
 
-See L<cpandb-meta> script.
+See L<cpandb-cpanmeta> script.
 
 =head1 DESCRIPTION
 
@@ -560,7 +637,7 @@ extracted/recorded are: dependencies (tables: C<files>, C<deps>).
 
 =head2 deps_cpan_meta(%args) -> [status, msg, result, meta]
 
-{en_US Query dependency from CPAN::SQLite database}.
+Query dependency from CPAN::SQLite database.
 
 Arguments ('*' denotes required arguments):
 
@@ -568,29 +645,27 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
+Location of your local CPAN mirror, e.g. /path/to/cpan.
 
 =item * B<db_dir> => I<str>
 
-{en_US Database directory, defaults to your CPAN home}.
+Database directory, defaults to your CPAN home.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-{en_US Database name}.
+Database name.
 
 =item * B<include_core> => I<bool> (default: 0)
 
-{en_US Include Perl core modules}.
+Include Perl core modules.
 
 =item * B<level> => I<int> (default: 1)
 
-{en_US Recurse for a number of levels (-1 means unlimited)}.
-
-=item * B<module>* => I<str>
+Recurse for a number of levels (-1 means unlimited).
 
 =item * B<perl_version> => I<str> (default: "v5.18.4")
 
-{en_US Set base Perl version for determining core modules}.
+Set base Perl version for determining core modules.
 
 =item * B<phase> => I<str> (default: "runtime")
 
@@ -612,7 +687,7 @@ Return value:  (any)
 
 =head2 index_cpan_meta(%args) -> [status, msg, result, meta]
 
-{en_US Index CPAN Meta information into CPAN::SQLite database}.
+Index CPAN Meta information into CPAN::SQLite database.
 
 Arguments ('*' denotes required arguments):
 
@@ -620,15 +695,15 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
+Location of your local CPAN mirror, e.g. /path/to/cpan.
 
 =item * B<db_dir> => I<str>
 
-{en_US Database directory, defaults to your CPAN home}.
+Database directory, defaults to your CPAN home.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-{en_US Database name}.
+Database name.
 
 =back
 
@@ -646,7 +721,7 @@ Return value:  (any)
 
 =head2 revdeps_cpan_meta(%args) -> [status, msg, result, meta]
 
-{en_US Query reverse dependencies from CPAN::SQLite database}.
+Query reverse dependencies from CPAN::SQLite database.
 
 Arguments ('*' denotes required arguments):
 
@@ -654,15 +729,15 @@ Arguments ('*' denotes required arguments):
 
 =item * B<cpan>* => I<str>
 
-{en_US Location of your local CPAN mirror, e.g. /path/to/cpan}.
+Location of your local CPAN mirror, e.g. /path/to/cpan.
 
 =item * B<db_dir> => I<str>
 
-{en_US Database directory, defaults to your CPAN home}.
+Database directory, defaults to your CPAN home.
 
 =item * B<db_name> => I<str> (default: "cpandb.sql")
 
-{en_US Database name}.
+Database name.
 
 =item * B<module>* => I<str>
 
@@ -681,7 +756,7 @@ Return value:  (any)
 
 =head1 SEE ALSO
 
-Front-end for this module: L<cpandb-meta>.
+Front-end for this module: L<cpandb-cpanmeta>.
 
 L<CPAN::SQLite> and its front-end L<cpandb>.
 
